from flask import current_app
from jinja2 import Markup
from xml.dom import minidom
import io

from . import icons

class Feather(object):
    def __init__(self, app=None):
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        if not hasattr(app,'extensions'):
            app.extension= {}
        app.extensions['feather']=_feather
        app.context_processor(self.context_processor)
    
    @staticmethod
    def context_processor():
        return {
            'feather':current_app.extensions['feather']
        }
    def create(self, context):
        pass

class _feather(object):
    @staticmethod
    def icon(icon_name, **kwargs):
        """Attempt to render the icon icon_name with attributes as listed.

        Args:
            icon_name (string): The name of the feather icon
        """
        icon_name = icon_name.replace('-','_')
        if not icon_name:
            return ''
        doc = minidom.parseString(getattr(icons, icon_name))
        for attr, val in kwargs.items():
            attr = attr.replace('_','-')
            doc.documentElement.setAttribute(attr, str(val))
        writer = io.StringIO()
        SVGDocument(doc).writexml(writer)
        return Markup(writer.getvalue())

class SVGDocument:
    def __init__(self, doc: minidom.Document):
        self._doc = doc

    @property
    def doc(self) -> minidom.Document:
        return self._doc

    def writexml(self, writer, indent="", addindent="", newl=""):
        """Ignore the xml tag"""
        for node in self.doc.childNodes:
            node.writexml(writer, indent, addindent, newl)


# -*- coding: utf-8 -*-

__pkg__ = 'Flask-Feather'
__url__ = 'https://github.com/thetomcat/Flask-Feather'
__info__ = 'Insert feather icons into jinja2 templates'
__author__ = 'Tom Vos'
__email__ = 'tjvos1@gmail.com'
__version__ = '0.0.5'

from .extension import Feather


import os
import re
from os import path

from setuptools import setup, find_packages
from setuptools.command.build_py import build_py


class BuildIconsCommand(build_py):
    """Custom Build Icons Command
    """
    def run(self):
        # First copy all files
        super(BuildIconsCommand, self).run()
        build_file = path.join('build', 'lib', 'flask_feather','icons.py')
        icon_dir = path.join('feather', 'icons')
        files = [f for f in os.listdir(icon_dir)
                 if path.isfile(path.join(icon_dir, f))]
        with open(build_file, 'w') as icons:
            # Open the build file once, then iterate over all icons
            for file in files:
                icon_name = str(file.split('.')[0]).replace('-', '_')
                with open(path.join(icon_dir, file), 'r') as icon:
                    svg = icon.read()
                # Modify the svg, remove line and spaces
                svg = re.sub(r'\n', r' ', svg)
                svg = re.sub(r'\s+', r' ', svg)
                svg = svg.replace('> <', '><').replace(' />', '/>')
                # Add to the build file
                icons.write("%s = \'%s\'%s" % (icon_name, svg, os.linesep))


with open("README.md", "r") as fh:
    long_description = fh.read()

install_requires = [
    'flask>=1.1',
    'Jinja2'
]

testing_extras = []

setup(
    name='flask-feather',
    version='0.0.5',
    author='Tom Vos',
    author_email='tjvos1@gmail.com',
    cmdclass={
        'build_py': BuildIconsCommand
    },
    description='A simple Tag to implement Feather Icons in Flask',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/thetomcat/flask-feather',
    # py_modules=["flask_feather"],
    package_dir= {'':'src'},
    packages=find_packages(where='src'),
    include_package_data=True,
    install_requires=install_requires,
    extras_require={
        'dev': ["pytest>=3.7", 'twine>=3.2', 'check-manifest'],
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Flask',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        "Operating System :: OS Independent",
    ]
)




from flask import Flask, render_template_string
from flask_feather import Feather

app = Flask(__name__)
feather = Feather(app)

def test_simple():
    with app.app_context():
        icon = render_template_string("{{ feather.icon('user-plus') }}")
        assert '<svg ' in icon
        assert '</svg>' in icon
        
        
def test_attr():
    with app.app_context():
        assert 'width="20"' in render_template_string("{{ feather.icon('aperture', width=20) }}")

def test_mod_attr():
    with app.app_context():
        assert 'stroke-width="20"' in render_template_string("{{ feather.icon('aperture', stroke_width=20) }}")